#!/usr/bin/env python
# -*- coding: utf-8 -*-

def toDXF(ixyz, dxf_file, layer='Puntos', layer_id='Puntos_ID'):
    """
    Toma una lista de coordenadas xyz = [(x1,y1,z1), (x2,y2,z2), ... ] y genera
    un archivo formato DXF con nombre dxf_file con los puntos correspondientes
    a esas coordenadas en la layer 'Puntos'.
    """
    pf = open(dxf_file, 'w')
    pf.write('  0\nSECTION\n  2\nENTITIES\n')
    for p in ixyz:
        pf.write('  0\nPOINT\n  8\n' + layer + 
                 '\n 10\n' + str(p[1]) + 
                 '\n 20\n' + str(p[2]) + 
                 '\n 30\n' + str(p[3]) +
                 '\n  0\nTEXT\n  8\n' + layer_id +
                 '\n  1\n' + str(p[0]) + 
                 '\n 10\n' + str(p[1]) + 
                 '\n 20\n' + str(p[2]) + 
                 '\n 30\n' + str(p[3]) +
                 '\n 40\n10\n')
    pf.write('  0\nENDSEC\n')
    pf.write('  0\nEOF\n')
    pf.close()

def coord2dict(coord_file, sep=','):
    """
    Toma un archivo con id, x, y[, z] y devuelve un diccionario
    {id1: (x1,y1,z1), id2: (x2,y2,z2), ...}
    """
    file_in = open(coord_file, 'r')
    ixyz = {}
    for entry in file_in:
        if not entry.startswith('#') and entry.strip() != '' \
          and entry.find(sep) != -1:
            coord = entry.strip().split(sep)
            point_id = coord[0]
            x = float(coord[1])
            y = float(coord[2])
            z = 0.0 if len(coord) < 4 else coord[3]
            ixyz.update({point_id: (x, y, z)})
    file_in.close()
    return ixyz

def coord2list(coord_file, sep=','):
    """
    Toma un archivo con id, x, y[, z] y devuelve una lista
    [(id1,x1,y1,z1), (id2,x2,y2,z2), ...]
    """
    file_in = open(coord_file, 'r')
    ixyz = []
    for entry in file_in:
        if not entry.startswith('#') and entry.strip() != '' \
          and entry.find(sep) != -1:
            coord = entry.strip().split(sep)
            point_id = coord[0]
            x = float(coord[1])
            y = float(coord[2])
            z = 0.0 if len(coord) < 4 else coord[3]
            ixyz.append((point_id, x, y, z))
    file_in.close()
    return ixyz

def coord2dxf(coord_file, dxf_file=''):
    """
    Toma un archivo con id, x, y[, z] y devuelve un archivo formato DXF con
    nombre dxf_file con los puntos correspondientes a esas coordenadas en la
    layer (capa) 'Puntos'.
    """
    dxf_file = dxf_file if dxf_file != '' else coord_file + '.dxf'
    toDXF(coord2list(coord_file), dxf_file)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Pasos para transformar proy a geod/proy/GK y exportar a DXF

from geod_proy import *
from toDXF import *


#
# 1) Configurar Proyecciones usadas
#

# 1.1) Proyección Mercator Transversal cualquiera.
lat_orig = gms2gyf(-33,52)
merid_c = gms2gyf(-61,14)
pserapio = config_proy(lat_orig, merid_c)

# 1.2) Gauss-Kruger Faja 5
gk_faja5 = proyGK(5)


#
# 2) Transformar entre geodésicas y proyectadas
#

# 2.1) El proceso es proy -> geod -> proy -> geod. Si se comparan los archivos
#      de salida, deberían ser iguales entre ambas "proy" y ambas "geod".

# proy -> proy.geod
proy2geod('coord/proy', pserapio)
# proy.geod -> proy.geod.proy
geod2proy('coord/proy.geod', pserapio)
# proy.geod.proy -> proy.geod.proy.geod
proy2geod('coord/proy.geod.proy', pserapio)


#
# 3) Transformar entre geodésicas y proyectadas (GK-Faja5)
#

# 3.1) El proceso es geod -> gk5 -> geod -> gk5. Si se comparan los archivos de
#      salida, deberían ser iguales entre ambas "gk5" y ambas "geod".

# proy.geod -> proy.geod.gk5
geod2proy('coord/proy.geod', gk_faja5, 'gk5')
# proy.geod.gk5 -> proy.geod.gk5.geod
proy2geod('coord/proy.geod.gk5', gk_faja5)
# proy.geod.gk5.geod -> proy.geod.gk5.geod.gk5
geod2proy('coord/proy.geod.gk5.geod', gk_faja5, 'gk5')
# proy.geod.gk5.geod.gk5 -> proy.geod.gk5.geod.gk5.geod
proy2geod('coord/proy.geod.gk5.geod.gk5', gk_faja5)


#
# 4) Exportar a DXF
#

# Sólo tiene sentido mandar a DXF las coordenadas proyectadas.
coord2dxf('coord/proy')
coord2dxf('coord/proy.geod.proy')
coord2dxf('coord/proy.geod.gk5')
coord2dxf('coord/proy.geod.gk5.geod.gk5')




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pyproj
from gms_gyf import *

def config_proy(latitud_de_origen, meridiano_central, falso_este=0,
    falso_norte=0, factor_de_escala=1, datum='WGS84'):
    """Crea una proyeccion de tipo mercator transversal, con los siguientes
    parametros:
      latitud_de_origen     Latitud de origen
      meridiano_central     Meridiano central
      falso_este            Falso Este (0 por defecto, en metros)
      falso_norte           Falso Norte (0 por defecto, en metros)
      factor_de_escala      Factor de escala (1 por defecto)
      datum                 Datum, 'WGS84' por defecto (ver variable pj_ellps
      para otros datums posibles, ej: 'GRS80')
    """
    pmerct = pyproj.Proj(
        proj='tmerc',
        lat_0=latitud_de_origen,
        lon_0=meridiano_central,
        k=factor_de_escala,
        x_0=falso_este,
        y_0=falso_norte,
        ellps=datum)
    return pmerct


def proyGK(faja, datum='WGS84'):
    lat = -90
    lon = -(72 - (faja - 1) * 3)
    fe = faja * 1000000 + 500000
    fn = 0
    k = 1
    dat = datum
    return config_proy(lat, lon, fe, fn, k, dat)


def proy2geod(archivo_de_coordenadas, proyeccion, ext='geod', separador=','):
    """
    Wrapper de geod_proy con valores inv=True y file_ext='geod'
    """
    inv = True
    geod_proy(archivo_de_coordenadas, proyeccion, ext, inv, separador)


def geod2proy(archivo_de_coordenadas, proyeccion, ext='proy', separador=','):
    """
    Wrapper de geod_proy con valores inv=False y file_ext='proy'
    """
    inv = False
    geod_proy(archivo_de_coordenadas, proyeccion, ext, inv, separador)


def geod_proy(coord_file, proj, file_ext, inv, sep=','):
    """
    Transforaciones entre coordenadas geodesicas y planas proyectadas.
    Paramatros requeridos:
      coord_file: nombre del archivo de coordenadas
      proj:       proyección obtenida con la función 'config_proy'
      file_ext:   la extensión del archivo de salida
      inv (Bool): indica si la transformación va hacia un lado o hacia el otro
                  True: proy -> geod
                  False: geod -> proy
      sep: separador columnas del archivo de coordenadas (por defecto: ',')
    """
    # abrir los archivos de entrada y salida
    file_in = open(coord_file, 'r')
    file_out = open(coord_file + '.' + file_ext, 'a')
    # imprimir encabezado con info de proyección
    file_out.write('# Info de la proyeccion:\n#\t' + proj.srs + '\n')
    if inv:
        file_out.write('#id, lon (º \' "),\tlat (º \' ")\n')
    else:
        file_out.write('#id, E (m),\tN (m)\n')
    # recorrer archivo, transformar coordenadas y escribir archivo de salida
    for entry in file_in:
        if not entry.startswith('#') and entry.strip() != '' \
        and entry.find(sep) != -1:
            coord_id, coord_in1, coord_in2 = entry.split(sep)
            # tratamiento especial previo a la transformación
            if not inv:
                g1, m1, s1 = coord_in1.strip().split(' ')
                g2, m2, s2 = coord_in2.strip().split(' ')
                coord_in1 = gms2gyf(int(g1), int(m1), float(s1))
                coord_in2 = gms2gyf(int(g2), int(m2), float(s2))
            # transformación de las coordenadas
            coord_out1, coord_out2 = proj(float(coord_in1), float(coord_in2), inverse=inv)
            # tratamiento especial posterior a la transformación
            if inv:
                g1, m1, s1 = gyf2gms(coord_out1)
                g2, m2, s2 = gyf2gms(coord_out2)
                coord_out1 = str(g1) + ' ' + str(m1) + ' ' + str(s1)
                coord_out2 = str(g2) + ' ' + str(m2) + ' ' + str(s2)
            else:
                coord_out1 = round(coord_out1, 3)
                coord_out2 = round(coord_out2, 3)
            # escribir al archivo de salida las coordenadas transformadas
            file_out.write(str(coord_id) + sep + str(coord_out1) \
              + sep + str(coord_out2) + '\n')
    # cerrar los archivos
    file_in.close()
    file_out.close()




#!/usr/bin/python
# -*- coding: UTF-8 -*-

from distutils.core import setup


with open('requirements.txt', 'r') as fh:
    dependencies = [l.strip() for l in fh]


setup(
    name='agrimpy',
    version='0.0.3',
    author='Santiago Pestarini',
    author_email='santiago@pestarini.com.ar',
    packages=['agrimpy'],
    package_data={'agrimpy': ['coord/*']},
    url='http://pypi.python.org/pypi/agrimpy/',
    license='LICENSE.txt',
    description='Algunas operaciones con coordenadas geodésicas.',
    long_description=open('README.rst').read(),
    install_requires=dependencies,
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Funciones de conversión entre (grados, minutos, segundos) sexagesimales y
# grados y fracción.

from math import copysign, trunc


def sign(x):
    return copysign(1, x)


def gms2gyf(grados, minutos=0, segundos=0):
    """Conversion (gg,mm,ss.sssss) --> (g.frac)
    Por ejemplo:
        gms2gyf(-61,30)  = -61.5
        gms2gyf(0,-45)   = -0.75
        gms2gyf(-33)     = -33.0"""
    s = sign(grados)
    signo = 1 if s == 0 else s
    return signo * (signo * grados + minutos / 60.0 + segundos / 3600.0)


def gyf2gms(gyf):
    """Conversion (g.frac) --> (gg,mm,ss.sssss)
    Por ejemplo:
        gyf2gms(-61.5) = (-61,30,0.0)
        gyf2gms(-0.75) = (0,-45,0.0)"""
    # hacer las cuentas
    g = trunc(gyf)
    m = (gyf - g) * 60
    s = (m - trunc(m)) * 60
    # preparar la presentación
    g = int(g)
    m = int(m) if g == 0 else int(abs(m))
    s = abs(s) if m != 0 and g != 0 else s
    return (g, m, s)


def gyf2gms_list(gyf_list):
    """Conversion (g0.ggg,g1.ggg) --> ((g0,mm,ss.sss),(g1,mm,ss.sss)) masiva.
    Por ejemplo: gyf_list = [(-61.5,-31.456),(-63.456,31.3333),...]"""
    gms_list = []
    for i in gyf_list:
        gms_list.append((gyf2gms(i[0]), gyf2gms(i[1])))
    return gms_list


def gms2gyf_list(gms_list):
    """Conversion ((g0,mm,ss.sss),(g1,mm,ss.sss)) --> (g0.ggg,g1.ggg) masiva.
    Por ejemplo:
    gms_list = [((-61,30),(-31,45)),((-63,45),(31,20))]"""
    gyf_list = []
    for i in gms_list:
        gyf_list.append((gms2gyf(*i[0]), gms2gyf(*i[1])))
    return gyf_list

